#include <DS3231.h>
#include <TMCStepper.h>
#include <Adafruit_SSD1306.h>
#include <EEPROM.h>

#define PIN_MV_ENABLE          8 // Enable
#define PIN_MV_DIR             12 // Direction
#define PIN_MV_STEP            10 // Step
#define PIN_MV_SWITCH          18 //switch

#define PIN_MH_ENABLE          2 // Enable
#define PIN_MH_DIR             6 // Direction
#define PIN_MH_STEP            4 // Step
#define PIN_MH_SWITCH          22 //switch

#define PUMP_VERTICAL          1
#define PUMP_HORIZONTAL        2

#define SW_RX               63 // TMC2208/TMC2224 SoftwareSerial receive pin
#define SW_TX               40 // TMC2208/TMC2224 SoftwareSerial transmit pinf
#define DRIVER_ADDRESS      0b00 // TMC2209 Driver address according to MS1 and MS2
#define R_SENSE             0.11f // Match to your driver, SilentStepStick series use 0.11

TMC2209Stepper driver(SW_RX, SW_TX, R_SENSE, DRIVER_ADDRESS);

#define STEPPER_LOW_SPEED     400
#define STEPPER_MEDIUM_SPEED  300
#define STEPPER_HIGH_SPEED    200


void setup()
{
  Serial.begin(115200);  
    
  
  pinMode(PIN_MV_ENABLE, OUTPUT);
  pinMode(PIN_MV_STEP, OUTPUT);
  pinMode(PIN_MV_DIR, OUTPUT);
  pinMode(PIN_MV_SWITCH, INPUT);

  pinMode(PIN_MH_ENABLE, OUTPUT);
  pinMode(PIN_MH_STEP, OUTPUT);
  pinMode(PIN_MH_DIR, OUTPUT);
  pinMode(PIN_MH_SWITCH, INPUT);

  
  digitalWrite(PIN_MV_ENABLE, HIGH);  // Enable driver in hardware
  digitalWrite(PIN_MH_ENABLE, HIGH);  // Enable driver in hardware

  driver.begin();                 //  SPI: Init CS pins and possible SW SPI pins
                                  // UART: Init SW UART (if selected) with default 115200 baudrate
  driver.toff(5);                 // Enables driver in software
  driver.rms_current(800);        // Set motor RMS current
  driver.microsteps(16);          // Set microsteps to 1/16th

  driver.pwm_autoscale(true);     // Needed for stealthChop
  findZeroPosition(PUMP_HORIZONTAL);
  findZeroPosition(PUMP_VERTICAL);
}

void loop() 
{
  turn(PUMP_HORIZONTAL, 5, false, STEPPER_MEDIUM_SPEED);
  turn(PUMP_VERTICAL, 5, false, STEPPER_MEDIUM_SPEED);
  findZeroPosition(PUMP_HORIZONTAL);
  findZeroPosition(PUMP_VERTICAL);
  delay(10000);
}

void findZeroPosition(int pumpID)
{
   int PIN_SWITCH;
   if(pumpID==PUMP_VERTICAL)
   {
      PIN_SWITCH = PIN_MV_SWITCH;
   }
   else
   if(pumpID==PUMP_HORIZONTAL)
   {
      PIN_SWITCH = PIN_MH_SWITCH;
   }
  if(digitalRead(PIN_SWITCH)==1) //if already triggered swtich - move the position back a bit
  {
    turn(pumpID, 1, false, STEPPER_MEDIUM_SPEED);
  }
  while(digitalRead(PIN_SWITCH)==0) //not found zero position yet
  {
    turn(pumpID, 0.05, true, STEPPER_MEDIUM_SPEED);
  }
}



void turn(int pumpID, float n_turns, bool up, int delay_microseconds)
{
  int PIN_ENABLE;
  int PIN_STEP;
  int PIN_DIR;
  
  if(pumpID==PUMP_VERTICAL)
  {
    PIN_ENABLE = PIN_MV_ENABLE;
    PIN_STEP = PIN_MV_STEP;
    PIN_DIR = PIN_MV_DIR;
  }
  else
  if(pumpID==PUMP_HORIZONTAL)
  {
    PIN_ENABLE = PIN_MH_ENABLE;
    PIN_STEP = PIN_MH_STEP;
    PIN_DIR = PIN_MH_DIR;
  }
  
  
  digitalWrite(PIN_ENABLE, LOW);
  if(up)
  {
    digitalWrite(PIN_DIR,HIGH);
  }
  else
  {
    digitalWrite(PIN_DIR,LOW);    
  }
  
  for (long i = (long)(1600.0*n_turns); i>0; i--)
  {
    digitalWrite(PIN_STEP, HIGH);
    delayMicroseconds(delay_microseconds);
    digitalWrite(PIN_STEP, LOW);
    delayMicroseconds(delay_microseconds);
  }
  digitalWrite(PIN_ENABLE, HIGH);
}
